@@ -1,13 +1,13 @@
 /* Copyright 2015-2016 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
 
-#include "wireguard.h"
 #include "hashtables.h"
 #include "peer.h"
-#include "crypto/siphash24.h"
 #include "noise.h"
+#include "crypto/siphash24.h"
+
 #include <linux/hashtable.h>
 
-static inline struct hlist_head *pubkey_bucket(struct pubkey_hashtable *table, const uint8_t pubkey[NOISE_PUBLIC_KEY_LEN])
+static inline struct hlist_head *pubkey_bucket(struct pubkey_hashtable *table, const uint8_t pubkey[static NOISE_PUBLIC_KEY_LEN])
 {
 	/* siphash24 gives us a secure 64bit number based on a random key. Since the bits are
 	 * uniformly distributed, we can then mask off to get the bits we need. */
@@ -36,7 +36,7 @@ void pubkey_hashtable_remove(struct pubkey_hashtable *table, struct wireguard_pe
 }
 
 /* Returns a strong reference to a peer */
-struct wireguard_peer *pubkey_hashtable_lookup(struct pubkey_hashtable *table, const uint8_t pubkey[NOISE_PUBLIC_KEY_LEN])
+struct wireguard_peer *pubkey_hashtable_lookup(struct pubkey_hashtable *table, const uint8_t pubkey[static NOISE_PUBLIC_KEY_LEN])
 {
 	struct wireguard_peer *iter_peer, *peer = NULL;
 	rcu_read_lock();
@@ -61,7 +61,6 @@ static inline struct hlist_head *index_bucket(struct index_hashtable *table, con
 void index_hashtable_init(struct index_hashtable *table)
 {
 	get_random_bytes(table->key, SIPHASH24_KEY_LEN);
-	atomic64_set(&table->counter, 0);
 	hash_init(table->hashtable);
 	spin_lock_init(&table->lock);
 }
@@ -69,7 +68,7 @@ void index_hashtable_init(struct index_hashtable *table)
 __le32 index_hashtable_insert(struct index_hashtable *table, struct index_hashtable_entry *entry)
 {
 	struct index_hashtable_entry *existing_entry;
-	uint64_t counter;
+	uint64_t rand;
 
 	spin_lock(&table->lock);
 	hlist_del_init_rcu(&entry->index_hash);
@@ -79,8 +78,8 @@ __le32 index_hashtable_insert(struct index_hashtable *table, struct index_hashta
 
 search_unused_slot:
 	/* First we try to find an unused slot, randomly, while unlocked. */
-	counter = atomic64_inc_return(&table->counter);
-	entry->index = (__force __le32)siphash24((uint8_t *)&counter, sizeof(counter), table->key);
+	rand = get_random_long();
+	entry->index = (__force __le32)siphash24((uint8_t *)&rand, sizeof(rand), table->key);
 	hlist_for_each_entry_rcu(existing_entry, index_bucket(table, entry->index), index_hash) {
 		if (existing_entry->index == entry->index)
 			goto search_unused_slot; /* If it's already in use, we continue searching. */
