@@ -3,14 +3,16 @@
 #ifndef PEER_H
 #define PEER_H
 
-#include "wireguard.h"
 #include "noise.h"
 #include "cookie.h"
+
 #include <linux/types.h>
 #include <linux/netfilter.h>
 #include <linux/spinlock.h>
 #include <linux/kref.h>
 
+struct wireguard_device;
+
 struct wireguard_peer {
 	struct wireguard_device *device;
 	struct sockaddr_storage endpoint_addr;
@@ -39,7 +41,7 @@ struct wireguard_peer {
 	uint64_t internal_id;
 };
 
-struct wireguard_peer *peer_create(struct wireguard_device *wg, const u8 public_key[NOISE_PUBLIC_KEY_LEN]);
+struct wireguard_peer *peer_create(struct wireguard_device *wg, const u8 public_key[static NOISE_PUBLIC_KEY_LEN]);
 
 struct wireguard_peer *peer_get(struct wireguard_peer *peer);
 void peer_put(struct wireguard_peer *peer);
