@@ -1,9 +1,9 @@
 /* Copyright 2015-2016 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
 
-#include "wireguard.h"
 #include "timers.h"
-#include "packets.h"
 #include "device.h"
+#include "peer.h"
+#include "packets.h"
 
 enum {
 	KEEPALIVE = 10 * HZ,
@@ -18,6 +18,12 @@ enum {
  * Timer for, if enabled, sending an empty authenticated packet every user-specified seconds
  */
 
+/* This rounds the time down to the closest power of two of the closest quarter second. */
+static inline unsigned long slack_time(unsigned long time)
+{
+	return time & ~(roundup_pow_of_two(HZ / 4) - 1);
+}
+
 static void expired_retransmit_handshake(unsigned long ptr)
 {
 	struct wireguard_peer *peer = (struct wireguard_peer *)ptr;
@@ -142,7 +148,7 @@ void timers_ephemeral_key_created(struct wireguard_peer *peer)
 void timers_any_authenticated_packet_traversal(struct wireguard_peer *peer)
 {
 	if (peer->persistent_keepalive_interval && likely(peer->timer_persistent_keepalive.data))
-		mod_timer(&peer->timer_persistent_keepalive, jiffies + peer->persistent_keepalive_interval);
+		mod_timer(&peer->timer_persistent_keepalive, slack_time(jiffies + peer->persistent_keepalive_interval));
 }
 
 void timers_init_peer(struct wireguard_peer *peer)
@@ -154,12 +160,10 @@ void timers_init_peer(struct wireguard_peer *peer)
 	init_timer(&peer->timer_send_keepalive);
 	peer->timer_send_keepalive.function = expired_send_keepalive;
 	peer->timer_send_keepalive.data = (unsigned long)peer;
-	set_timer_slack(&peer->timer_send_keepalive, HZ / 4);
 
 	init_timer(&peer->timer_new_handshake);
 	peer->timer_new_handshake.function = expired_new_handshake;
 	peer->timer_new_handshake.data = (unsigned long)peer;
-	set_timer_slack(&peer->timer_new_handshake, HZ / 4);
 
 	init_timer(&peer->timer_kill_ephemerals);
 	peer->timer_kill_ephemerals.function = expired_kill_ephemerals;
@@ -168,7 +172,6 @@ void timers_init_peer(struct wireguard_peer *peer)
 	init_timer(&peer->timer_persistent_keepalive);
 	peer->timer_persistent_keepalive.function = expired_send_persistent_keepalive;
 	peer->timer_persistent_keepalive.data = (unsigned long)peer;
-	set_timer_slack(&peer->timer_persistent_keepalive, max_t(int, HZ / 2, peer->persistent_keepalive_interval / 256));
 
 	INIT_WORK(&peer->clear_peer_work, queued_expired_kill_ephemerals);
 }
